package org.demo;

import org.apache.camel.Exchange;

import java.io.*;
import java.util.concurrent.CompletableFuture;

public class Helper {
	
	public CompletableFuture<Void> asyncProcessStream(Exchange exchange) {
        return CompletableFuture.runAsync(() -> {
            try {
                processStream(exchange);
            } catch (Exception e) {
                exchange.setException(e);
            }
        });
    }

	//reads the full stream and counts the number of bytes
	public void processStream(Exchange exchange) throws Exception
	{
		InputStream is = exchange.getIn().getBody(InputStream.class);

		byte[] buffer = new byte[1024];
		long read = 0;
		long total = 0;
		while ((read = is.read(buffer, 0, buffer.length)) != -1) {
			total += read;
		}

		// System.out.println("Stream length (in bytes): "+ total);

		exchange.getIn().setBody(new Long(total));
	}

	public void prepareStream(Exchange exchange) throws Exception
	{
		PipedOutputStream pos = new PipedOutputStream();
		PipedInputStream pis = new PipedInputStream(pos);

		exchange.getIn().setBody(pis);

		streamWriter sw = new streamWriter(pos);
		sw.start();
	}


	//Artificially generates big streams (2 GigaBytes)
  	private class streamWriter extends Thread {

	  	PipedOutputStream pos;

	  	public streamWriter(PipedOutputStream stream)
	  	{
	  		pos = stream;
	  	}

	    public void run()
	    {

	    	byte[] response = "hello world stream\n".getBytes();

			long kb = 1024;
			long mb = 1024*kb;
			long gb = 1024*mb;

			long limit = 2*gb;
			long count = 0;

			// System.out.println("count = "+count+", limit = "+limit);

			try
			{
				while(count < limit)
				{
					pos.write(response);
					pos.flush();
					count+=response.length;
				}

				pos.close();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}

			// System.out.println("thread done.");
	    }
	}



}
